/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020                                    ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_CONVERSION_OPENEXCHANGERATESSOURCE_H_
#define DRN_CONVERSION_OPENEXCHANGERATESSOURCE_H_

#include <QByteArray>
#include <QObject>
#include <QString>

#include "CurrencySource.hpp"
#include "conversion_export.h"


class QNetworkReply;

namespace pecunia
{
namespace currency
{

enum class Codes;

}}

namespace drn
{
namespace network
{

struct AccessManager;

}
namespace conversion
{

/**
 * @brief Manages the details of fetching conversion rates between two currencies from the
 * Open Exchange Rates, https://openexchangerates.org/ .
 */
class DRN_CONVERSION_EXPORT OpenExchangeRatesSource : public QObject, public CurrencySource
{
	Q_OBJECT

	network::AccessManager& networkManager_;
	QString appId_;
	QNetworkReply* reply_;
	QByteArray data_;

private slots:
	void replyFinished();

public:
	/**
	 * @brief Full initialisation constructor.
	 *
	 * @param networkManager The object through which the network is accessible for sending requests.
	 * @param appId The application identifier for the OER servers.
	 * @param parent The parent widget of the object.
	 */
	OpenExchangeRatesSource(
		network::AccessManager& networkManager,
		const QString& appId,
		QObject* parent = nullptr
	);
	~OpenExchangeRatesSource();
	/**
	 * @brief Synchronously retrieves the conversion from one currency code to another.
	 *
	 * @exception ReplyError When it fails to during the reply of a conversion request.
	 * @exception ParsingError When the data returned contains an error.
	 *
	 * @param from The currency to convert from.
	 * @param to The currency to convert to.
	 *
	 * @return The ratio that when multiplied will take the value of "from" to the value of "to".
	 */
	double fetch(
		const pecunia::currency::Codes& from,
		const pecunia::currency::Codes& to
	) override;
	/**
	 * @brief The application identifier for the OER servers.
	 */
	const QString& applicationId() const;
};

}}

#endif
