/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020                                    ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_FILE_STORAGE_FILESTORAGEERRORS_H_
#define DRN_FILE_STORAGE_FILESTORAGEERRORS_H_

#include <QString>

#include <error-handling/Error.h>
#include "file-storage_export.h"


namespace drn
{
namespace file_storage
{

struct DRN_FILE_STORAGE_EXPORT BudgetFileError : error_handling::Error
{
	BudgetFileError(
		const QString& errorString,
		const qint64& lineNumber,
		const qint64& columnNumber,
		const QString& message,
		const std::exception& root = {}
	);
};
struct DRN_FILE_STORAGE_EXPORT AttributeError: BudgetFileError
{
	AttributeError(
		const QString& tagName,
		const QLatin1String& attributeName,
		const QString& errorString,
		const qint64& lineNumber,
		const qint64& columnNumber,
		const QString& message,
		const std::exception& root = {}
	);
};
struct DRN_FILE_STORAGE_EXPORT BinaryFileError : error_handling::Error
{
	BinaryFileError(
		const QString& fileName,
		const QString& message,
		const std::exception root = {}
	);
};

}}

#endif
