/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020                                    ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_FILE_STORAGE_QTCONVERSIONS_H_
#define DRN_FILE_STORAGE_QTCONVERSIONS_H_

#include <set>

#include "file-storage_export.h"


namespace date
{

class year_month_day;

}

namespace pecunia
{
namespace currency
{

class Money;

}}

class QDate;
class QString;

namespace drn
{
namespace accounting
{

class AccountCode;
class AccountNumber;

}
namespace budget_accounting
{

enum class BankAccountTypes;

}
namespace foundation
{

template<typename>
class Optional;

}
namespace file_storage
{

/**
 * @brief Converts a string representation of the account code into the strong type.
 *
 * @throw DataEntryError When the supplied string form of the account code is invalid.
 *
 * @param accountCode The account code to convert.
 *
 * @return The strong form of the account code.
 */
DRN_FILE_STORAGE_EXPORT accounting::AccountCode toAccountCode(const QString& accountCode);
/**
 * @brief Converts a string representation of the account code and parent into the strong type.
 *
 * @throw DataEntryError When the supplied string form of the account code is invalid.
 *
 * @param accountCode The account code to convert.
 * @param parentNumber (Optional) The number for the parent account.
 *
 * @return The strong form of the account code.
 */
DRN_FILE_STORAGE_EXPORT accounting::AccountCode toFullAccountCode(
	const QString& accountCode,
	const QString& parentNumber
);
/**
 * @brief Converts a structured representation of the account code number into a string.
 *
 * @param number The account code number to convert.
 *
 * @return The QString from of the account code number.
 */
DRN_FILE_STORAGE_EXPORT QString toQString(const accounting::AccountNumber& number);
/**
 * @brief Converts a structured representation of the account code into a string.
 *
 * @param code The account code to convert.
 *
 * @return The QString from of the account code.
 */
DRN_FILE_STORAGE_EXPORT QString toQString(const accounting::AccountCode& code);
/**
 * @brief Converts a date representation into a Qt representation.
 *
 * @param date The date library representation to convert.
 *
 * @return The Qt representation.
 */
DRN_FILE_STORAGE_EXPORT QDate toQDate(const date::year_month_day& date);
/**
 * @brief Converts a date representation into a Qt representation.
 *
 * @param date The Qt representation to convert.
 *
 * @return The date library representation.
 */
DRN_FILE_STORAGE_EXPORT date::year_month_day toYearMonthDay(const QDate& date);
DRN_FILE_STORAGE_EXPORT QString toQString(const pecunia::currency::Money& money);
DRN_FILE_STORAGE_EXPORT QString toQString(
	const foundation::Optional<accounting::AccountNumber>& number,
	const QString& emptyValue
);
DRN_FILE_STORAGE_EXPORT foundation::Optional<accounting::AccountNumber> toOptionalAccountNumber(
	const QString& number,
	const QString& emptyValue
);
DRN_FILE_STORAGE_EXPORT QString toQString(const budget_accounting::BankAccountTypes& type);
DRN_FILE_STORAGE_EXPORT budget_accounting::BankAccountTypes toBankAccountTypes(const QString& bankAccountType);
DRN_FILE_STORAGE_EXPORT std::set<QString> toQStringAccountNames(const std::set<accounting::AccountCode>& codes);

}}

#endif
