/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020                                    ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_FOUNDATION_ENUMERATIONMACROS_HPP
#define DRN_FOUNDATION_ENUMERATIONMACROS_HPP

#include <stdexcept>
#include <string>
#include <typeinfo>
#include <type_traits>


#define BEGIN_WEAK_TO_ENUMERATION(enumType)  \
	if ( ! std::is_enum<enumType>::value)  \
		throw std::logic_error{  \
			std::string{"Cannot convert to an enumeration from a weak type. "} +  \
			"The supplied strong type '" + typeid(enumType).name() + "' is not an enumeration."  \
		}

#define MAP_WEAK_TO_ENUMERATION(weakValue, enumType, enumValue)  \
	else if (weakValue == static_cast<std::underlying_type_t<enumType>>(enumValue))  \
		return enumValue

#define END_WEAK_TO_ENUMERATION(weakValue, enumType)  \
	else  \
		throw std::logic_error{  \
			"Failed to convert from the weak value '" + std::to_string(weakValue)  \
			+ "' to the strong type for '" + typeid(enumType).name() + "'."  \
		}

#define BEGIN_ENUMERATION_TO_WEAK(strongValue)  \
	switch (strong)  \
	{  \

#define MAP_ENUMERATION_TO_WEAK(enumValue)  \
	case enumValue:  \
		return static_cast<std::underlying_type_t<decltype(enumValue)>>(enumValue)

#define END_ENUMERATION_TO_WEAK(strongValue)  \
	default:  \
		throw std::logic_error{  \
			"Failed to convert from the strong value '"  \
			+ std::to_string(  \
				static_cast<std::underlying_type_t<decltype(strongValue)>>(strongValue)  \
			)  \
			+ "' to the weak type for '" + typeid(decltype(strongValue)).name() + "'."  \
		};  \
	}

#define BEGIN_ENUMERATION_TO_OTHER(strongValue)  \
	switch (strongValue)  \
	{  \

#define MAP_ENUMERATION_TO_OTHER(enumValue, otherValue)  \
	case enumValue:  \
		return otherValue;

#define END_ENUMERATION_TO_OTHER(strongValue, enumType)  \
	default:  \
		throw std::logic_error{  \
			"Failed to convert from the strong value '"  \
				+ std::to_string(static_cast<std::underlying_type_t<enumType>>(strongValue))  \
				+ "' to another type for '" + typeid(enumType).name() + "'."  \
		};  \
	}

#define BEGIN_OTHER_TO_ENUMERATION(enumType)  \
	if ( ! std::is_enum<enumType>::value)  \
		throw std::logic_error{  \
			std::string{"Cannot convert to an enumeration from another type. "}  \
				+ "The supplied strong type '" + typeid(enumType).name()  \
				+ "' is not an enumeration."  \
		}

#define MAP_OTHER_TO_ENUMERATION(weakValue, otherValue, enumValue)  \
	else if (weakValue == otherValue)  \
		return enumValue

#define END_OTHER_TO_ENUMERATION(weakValue, enumType)  \
	else  \
		throw std::logic_error{  \
			"Failed to convert from the weak value '" + weakValue  \
				+ "' to another type for '" + typeid(enumType).name() + "'."  \
		}

#endif
