/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020                                    ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef DRN_FOUNDATION_FLOATING_POINT_EQUAL_H_
#define DRN_FOUNDATION_FLOATING_POINT_EQUAL_H_

#include "foundation_export.h"


namespace drn
{
namespace foundation
{

/**
 * @brief Determines if a floating-point value is equal to zero within 1/1000000000000.
 *
 * @param number The value to test.
 *
 * @return True when the number is roughly equal to zero.
 */
DRN_FOUNDATION_EXPORT bool isZero(const float number);
/**
 * @brief Determines if two floating-point values are equal within 1/1000000000000.
 *
 * @note This should not be used to test that a value is equal to zero. Use isZero instead.
 *
 * @param lhs The left-hand side floating-point value.
 * @param rhs The right-hand side floating-point value.
 *
 * @return True when the left-hand side is roughly equal to the right-hand side.
 */
DRN_FOUNDATION_EXPORT bool isEqual(const float lhs, const float rhs);
/**
 * @brief Determines if a floating-point value is equal to zero within 1/1000000000000.
 *
 * @param number The value to test.
 *
 * @return True when the number is roughly equal to zero.
 */
DRN_FOUNDATION_EXPORT bool isZero(const double number);
/**
 * @brief Determines if two floating-point values are equal within 1/1000000000000.
 *
 * @note This should not be used to test that a value is equal to zero. Use isZero instead.
 *
 * @param lhs The left-hand side floating-point value.
 * @param rhs The right-hand side floating-point value.
 *
 * @return True when the left-hand side is roughly equal to the right-hand side.
 */
DRN_FOUNDATION_EXPORT bool isEqual(const double lhs, const double rhs);
/**
 * @brief Determines if a floating-point value is equal to zero within 1/1000000000000.
 *
 * @param number The value to test.
 *
 * @return True when the number is roughly equal to zero.
 */
DRN_FOUNDATION_EXPORT bool isZero(const long double number);
/**
 * @brief Determines if two floating-point values are equal within 1/1000000000000.
 *
 * @note This should not be used to test that a value is equal to zero. Use isZero instead.
 *
 * @param lhs The left-hand side floating-point value.
 * @param rhs The right-hand side floating-point value.
 *
 * @return True when the left-hand side is roughly equal to the right-hand side.
 */
DRN_FOUNDATION_EXPORT bool isEqual(const long double lhs, const long double rhs);

}}

#endif
